<?php

namespace XF2SEO\SEOPackage\ControllerPlugin;

use XF\Mvc\Entity\Entity;
use XF2SEO\SEOPackage\App;
use XF\Mvc\Reply\AbstractReply;
use XF\ControllerPlugin\AbstractPlugin;

class Metadata extends AbstractPlugin
{
    const INPUT_PREFIX_METADATA = 'metadata';

    public function actionMetadata(
        Entity $content,
        string $confirmUrl,
        string $returnUrl,
        array $breadcrumbs = []
    ): AbstractReply {
        list($contentType, $contentId) = App::getContentTypeId($content);

        /** @var \XF2SEO\SEOPackage\Entity\Metadata|null $metadata */
        $metadata = $this->finder('XF2SEO\SEOPackage:Metadata')
            ->where('content_type', $contentType)
            ->where('content_id', $contentId)
            ->fetchOne();

        if ($this->isPost()) {
            $input = $this->filter([
                self::INPUT_PREFIX_METADATA => 'array'
            ]);

            $this->saveMetadata($content, $input[self::INPUT_PREFIX_METADATA]);

            return $this->redirect($returnUrl);
        }

        return $this->view(
            'XF2SEO\SEOPackage:Metadata\Form',
            'seo_package_metadata_form',
            [
                'content' => $content,
                'metadata' => $metadata,
                'confirmUrl' => $confirmUrl,
                'breadcrumbs' => $breadcrumbs,
                'inputPrefix' => self::INPUT_PREFIX_METADATA,
                'contentType' => $contentType,
            ]
        );
    }

    public function saveMetadata(Entity $content, array $metadataData): \XF2SEO\SEOPackage\Entity\Metadata
    {
        list($contentType, $contentId) = App::getContentTypeId($content);

        /** @var \XF2SEO\SEOPackage\Entity\Metadata|null $metadata */
        $metadata = $this->finder('XF2SEO\SEOPackage:Metadata')
            ->where('content_type', $contentType)
            ->where('content_id', $contentId)
            ->fetchOne();
        if ($metadata === null) {
            /** @var \XF2SEO\SEOPackage\Entity\Metadata $metadata */
            $metadata = $this->em()->create('XF2SEO\SEOPackage:Metadata');
            $metadata->content_type = $contentType;
            $metadata->content_id = $contentId;
        }

        $metadata->metadata = $this->verifyMetadata($contentType, $metadataData);
        $metadata->save();

        return $metadata;
    }

    protected function verifyMetadata(string $contentType, array $metadata): array
    {
        $rules = [
            'noindex' => 'bool',
            'title' => 'str',
            'description' => 'str',
        ];
        $filtered = $this->app->inputFilterer()->filterArray($metadata, $rules);

        return array_replace($metadata, $filtered);
    }
}
