<?php

namespace XF2SEO\SEOPackage;

use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;
use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF2SEO\SEOPackage\DevHelper\SetupTrait;

class Setup extends AbstractSetup
{
    use SetupTrait;
    use StepRunnerInstallTrait;
    use StepRunnerUpgradeTrait;
    use StepRunnerUninstallTrait;

    public function installStep1(): void
    {
        $this->doCreateTables($this->getTables());
        $this->doAlterTables($this->getAlters());
    }

    public function uninstallStep1(): void
    {
        $this->doDropTables($this->getTables());
        $this->doDropColumns($this->getAlters());
    }

    public function upgrade1000100Step1(): void
    {
        $this->doCreateTables($this->getTables2());
    }

    public function upgrade1000200Step1(): void
    {
        $this->doAlterTables($this->getAlters1());
    }

    protected function getTables1(): array
    {
        return [
            'xf_seo_package_metadata' => function (Create $table) {
                $table->addColumn('content_type', 'varchar', 50);
                $table->addColumn('content_id', 'int');
                $table->addColumn('metadata', 'TEXT');

                $table->addPrimaryKey(['content_type', 'content_id']);
            },
            'xf_seo_package_keyword' => function (Create $table) {
                $table->addColumn('keyword_id', 'int')->autoIncrement();
                $table->addColumn('keyword', 'varchar', 100);
                $table->addColumn('replacement', 'varchar', 255);
                $table->addColumn('description', 'text');
                $table->addColumn('target_url', 'varchar', 255);

                $table->addColumn('options', 'text');
                $table->addColumn('active', 'tinyint', 3)->setDefault(0);
                $table->addColumn('created_date', 'int')->setDefault(0);

                $table->addKey('active');
                $table->addUniqueKey('keyword');
            }
        ];
    }

    protected function getTables2(): array
    {
        return [
            'xf_seo_package_redirect' => function (Create $table) {
                $table->addColumn('redirect_id', 'int')->autoIncrement();
                $table->addColumn('match_url', 'varchar', 255);
                $table->addColumn('target_url', 'varchar', 255);
                $table->addColumn('redirect_type', 'enum', ['temporary', 'permanently'])
                    ->setDefault('temporary');
                $table->addColumn('active', 'tinyint', 3)->setDefault(0);
            },
        ];
    }

    protected function getAlters1(): array
    {
        return [
            'xf_seo_package_keyword' => [
                'execute_order' => function (Alter $table) {
                    $table->addColumn('execute_order', 'int')
                        ->setDefault(0);
                    $table->dropIndexes('active');
                    $table->addKey(['active', 'execute_order']);
                }
            ]
        ];
    }
}
